@charset "UTF-8";

.w3-theme-l5 {
background-color : #ffffff !important;
color : #000000 !important;}

.w3-theme-l4 {
background-color : #fffffe !important;
color : #000000 !important;}

.w3-theme-l3 {
background-color : #fffffb !important;
color : #000000 !important;}

.w3-theme-l2 {
background-color : #fffef3 !important;
color : #665e00 !important;}

.w3-theme-l1 {
background-color : #fefddb !important;
color : #636003 !important;}

.w3-theme-d1 {
background-color : #665e00 !important;
color : #fbf892 !important;}

.w3-theme-d2 {
background-color : #aaa506 !important;
color : #ffffff !important;}

.w3-theme-d3 {
background-color : #716e04 !important;
color : #ffffff !important;}

.w3-theme-d4 {
background-color : #4b4903 !important;
color : #fbf89d !important;}

.w3-theme-d5 {
background-color : #323102 !important;
color : #fbf99d !important;}

.w3-theme-light {
background-color : #fdfbc8 !important;
color : #625f04 !important;}

.w3-theme-dark {
background-color : #bfba07 !important;
color : #ffffff !important;}

.w3-theme-action {
background-color : #605d04 !important;
color : #fbf89d !important;}

.w3-theme {
background-color : #fbf892 !important;
color : #000000 !important;}

.w3-text-theme {
color : #f6093f !important;}

.w3-border-theme {
border_color : #f6093f !important;}

.w3-hover-theme:hover {
background-color : #09f632 !important;
color : #ffffff !important;}

.w3-hover-text-theme:hover {
color : #099ef6 !important;}

.w3-hover-border-theme:hover {
border-color : #099ef6 !important;}

