@charset "UTF-8";

.w3-theme-l5 {
background-color : #ffffff !important;
color : #000000 !important;}

.w3-theme-l4 {
background-color : #ffffff !important;
color : #000000 !important;}

.w3-theme-l3 {
background-color : #ffffff !important;
color : #000000 !important;}

.w3-theme-l2 {
background-color : #ffffff !important;
color : #000000 !important;}

.w3-theme-l1 {
background-color : #ffffff !important;
color : #000000 !important;}

.w3-theme-d1 {
background-color : #aaaaaa !important;
color : #000000 !important;}

.w3-theme-d2 {
background-color : #717171 !important;
color : #ffffff !important;}

.w3-theme-d3 {
background-color : #4b4b4b !important;
color : #ffffff !important;}

.w3-theme-d4 {
background-color : #323232 !important;
color : #cccccc !important;}

.w3-theme-d5 {
background-color : #212121 !important;
color : #cccccc !important;}

.w3-theme-light {
background-color : #ffffff !important;
color : #000000 !important;}

.w3-theme-dark {
background-color : #808080 !important;
color : #000000 !important;}

.w3-theme-action {
background-color : #404040 !important;
color : #ffffff !important;}

.w3-theme {
background-color : #ffffff !important;
color : #000000 !important;}

.w3-text-theme {
color : #808080 !important;}

.w3-border-theme {
border_color : #808080 !important;}

.w3-hover-theme:hover {
background-color : #808080 !important;
color : #000000 !important;}

.w3-hover-text-theme:hover {
color : #808080 !important;}

.w3-hover-border-theme:hover {
border-color : #808080 !important;}

