@charset "UTF-8";

.w3-theme-l5 {
background-color : #fffefe !important;
color : #000000 !important;}

.w3-theme-l4 {
background-color : #fefbfb !important;
color : #000000 !important;}

.w3-theme-l3 {
background-color : #fdf3f3 !important;
color : #570f0f !important;}

.w3-theme-l2 {
background-color : #f9dadc !important;
color : #580e13 !important;}

.w3-theme-l1 {
background-color : #ee8f97 !important;
color : #000000 !important;}

.w3-theme-d1 {
background-color : #480b10 !important;
color : #f1a7ad !important;}

.w3-theme-d2 {
background-color : #30070b !important;
color : #f2a6ad !important;}

.w3-theme-d3 {
background-color : #200507 !important;
color : #f1a7ac !important;}

.w3-theme-d4 {
background-color : #150305 !important;
color : #f2a6ae !important;}

.w3-theme-d5 {
background-color : #0e0203 !important;
color : #f2a6ac !important;}

.w3-theme-light {
background-color : #e65763 !important;
color : #000000 !important;}

.w3-theme-dark {
background-color : #36080c !important;
color : #f2a6ad !important;}

.w3-theme-action {
background-color : #1b0406 !important;
color : #f2a6ad !important;}

.w3-theme {
background-color : #6c1018 !important;
color : #ffffff !important;}

.w3-text-theme {
color : #a821de !important;}

.w3-border-theme {
border_color : #a821de !important;}

.w3-hover-theme:hover {
background-color : #c9de21 !important;
color : #ffffff !important;}

.w3-hover-text-theme:hover {
color : #21de5c !important;}

.w3-hover-border-theme:hover {
border-color : #21de5c !important;}

